pub struct Rpc { /* private fields */ }Expand description
Thread-local RPC endpoint.
Implementations
Sourceimpl Rpc
impl Rpc
Sourcepub fn new(
nexus: &Arc<Nexus>,
id: RpcId,
nic: &str,
phy_port: u8,
) -> Pin<Box<Self>>
pub fn new( nexus: &Arc<Nexus>, id: RpcId, nic: &str, phy_port: u8, ) -> Pin<Box<Self>>
Create a new Rpc instance that is bound to a Nexus with a certain
ID and the current thread. Will operate on the specified port of the given
device. The given ID must be unique among all RPCs in the same Nexus.
The created Rpc instance is pinned on heap to prevent moving around.
Otherwise, it can invalidate pointers recorded in request handles.
Panics
- Panic if the given ID is already used.
- Panic if there is no such device or no such port.
Sourcepub fn set_handler<H, F>(self: &mut Pin<Box<Self>>, req_id: ReqType, handler: H)
pub fn set_handler<H, F>(self: &mut Pin<Box<Self>>, req_id: ReqType, handler: H)
Set the handler for the given request type.
The handler takes a request handle as argument, and must return a MsgBuf that
belongs to the same Rpc calling the handler (otherwise, panic).
Panics
Panic if there is already an Rpc created.
Sourcepub fn create_session(
&self,
remote_uri: impl ToSocketAddrs,
remote_rpc_id: RpcId,
) -> SessionHandle<'_>
pub fn create_session( &self, remote_uri: impl ToSocketAddrs, remote_rpc_id: RpcId, ) -> SessionHandle<'_>
Create a client session that connects to a remote Rpc instance.
The created session is not connected by default, and it will spend no
efforts on establishing the connection until you call
Session::connect() on it.
The returned object does not have ownership of the session, but is purely
a handle to it. You may first remember the session ID and drop the handle.
When needed, you can acquire a new handle by calling Rpc::get_session().
Sourcepub fn get_session(&self, sess_id: SessId) -> Option<SessionHandle<'_>>
pub fn get_session(&self, sess_id: SessId) -> Option<SessionHandle<'_>>
Return a handle to a session of the given ID.
Sourcepub fn alloc_msgbuf(&self, len: usize) -> MsgBuf
pub fn alloc_msgbuf(&self, len: usize) -> MsgBuf
Allocate a MsgBuf that can accommodate at least len bytes of
application data. The allocated MsgBuf will have an initial length
of len, but its content are uninitialized.
MsgBufs allocated by one Rpc instance can only be used in the same
Rpc instance. This is because different Rpcs have different RDMA
device contexts, and the MsgBuf’s lkey is bound to the device context.
Panics
Panic if buffer allocation fails.